package ai.pai.lensman.session;

import android.content.Context;
import android.content.Intent;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.Toast;

import com.android.common.utils.DeviceUtils;
import com.nostra13.universalimageloader.core.DisplayImageOptions;

import java.io.File;
import java.util.ArrayList;

import ai.pai.lensman.App;
import ai.pai.lensman.R;
import ai.pai.lensman.bean.PhotoBean;
import ai.pai.lensman.service.UploadService;
import ai.pai.lensman.utils.Constants;
import ai.pai.lensman.utils.ImageLoaderUtils;
import butterknife.BindView;
import butterknife.ButterKnife;

public class PhotoRecyclerAdapter extends RecyclerView.Adapter<PhotoRecyclerAdapter.MyViewHolder> {

    private int width;
    private Context context;
    private LayoutInflater mInflater;
    private  DisplayImageOptions options;
    private ArrayList<PhotoBean> photoList;

    public PhotoRecyclerAdapter(Context context){
        this.context = context;
        width = DeviceUtils.getScreenWidth(context);
        mInflater = LayoutInflater.from(context);
        options = ImageLoaderUtils.getOptions(R.drawable.default_img);
    }

    public synchronized void addPhotoBean(PhotoBean item){
        if(photoList==null){
            photoList = new ArrayList<>();
        }
        if(!photoList.contains(item)){
            photoList.add(0,item);
            notifyItemInserted(0);
        }
    }

    public synchronized void removePhotoAtIndex(int index){
        if(photoList==null || photoList.size()<=index){
            return;
        }
        photoList.remove(index);
        notifyItemRemoved(index);
    }

    @Override
    public MyViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        return  new MyViewHolder(mInflater.inflate(R.layout.photo_item,parent,false));
    }

    @Override
    public void onBindViewHolder(final MyViewHolder holder, final int position) {
        if(photoList==null){
            return;
        }
        final PhotoBean item = photoList.get(position);
        String path =  Constants.APP_IMAGE_DIR + File.separator+item.sessionId+File.separator
                +Constants.THUMBNAIL_DIR_NAME+File.separator+item.photoName;
        ImageLoaderUtils.displayLocalImage(path, holder.photo, options);
        int height = width*10/16;
        ViewGroup.LayoutParams lp=holder.photo.getLayoutParams();
        lp.width = width;
        lp.height = height;
        holder.photo.setLayoutParams(lp);
        if(item.uploadStatus == PhotoBean.UploadStatus.STATUS_ERROR){
            holder.errorLayout.setVisibility(View.VISIBLE);
            holder.retryImg.setOnClickListener(new View.OnClickListener(){
                @Override
                public void onClick(View view) {
                    Intent intent = new Intent(App.getAppContext(), UploadService.class);
                    intent.putExtra("photo",item);
                    App.getAppContext().startService(intent);
                    item.uploadStatus = PhotoBean.UploadStatus.STATUS_NO_BEGIN;
                    notifyItemChanged(position);
                    Toast.makeText(context,R.string.add_to_upload_queue,Toast.LENGTH_SHORT).show();
                }
            });
        }else{
            holder.errorLayout.setVisibility(View.GONE);
        }
    }

    @Override
    public int getItemCount() {
        if(photoList==null){
            return 0;
        }
        return photoList.size();
    }

    class MyViewHolder extends RecyclerView.ViewHolder{

        @BindView(R.id.iv_session_photo_item) ImageView photo;
        @BindView(R.id.layout_upload_fail) View errorLayout;
        @BindView(R.id.iv_upload_retry)  ImageView retryImg;

        public MyViewHolder(View view){
            super(view);
            ButterKnife.bind(this,view);
        }
    }
}

